{{
    I2C PPDB 16 Segment Display Driver using 2 PCA9555
    This is my first attempt at something usefull.

    Project: Using the Full PPDB

    Version: 1.4
             I couldn't figure out how to connect the 4x4 Matrix keypad to the PCA9555
             so I simply connected it to the propeller starting from P8, I was hoping
             to save 8 pins but until I learn a bit more spin, I wont be able to get
             it right.
             Connect the 4x4 Keypad (facing you) into P8 to P15, I modified the
             4x4 Keypad reader driver to read the keypad this way. 

    Version: 1.3
             Updated Comments
             Split "GetTime" & "ShowTime"
             Removed code in SN and renamed to "Simple_Numbers_CutDown.spin"
           * Changed Propeller Pin Connections
             Added Pushbutton0 from the PPDB to P2 (Displays Date)
             Removed unused local variables in DS1302.spin & pca9555.spin
             
    Version: 1.2
             Added support for the PPDB's DS1302 RTC

    Version: 1.1
             16 Segment Display Working 

    License: MIT

        Copyright (c) 2011 Jorge Joaquin Pareja, http://WhatsAvailable.org

        Permission is hereby granted, free of charge, to any person obtaining a
        copy of this software and associated documentation files (the
        "Software"), to deal in the Software without restriction, including
        without limitation the rights to use, copy, modify, merge, publish,
        distribute, sublicense, and/or sell copies of the Software, and to
        permit persons to whom the Software is furnished to do so, subject
        to the following conditions:

        The above copyright notice and this permission notice shall be included
        in all copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
        OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
        IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
        CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
        TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
        SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


    Bug-list:
             LED Flicker very noticable when Time is displayed    

    Credit for Driver Code (ds1302, pca9555, Simple Numbers) goes to:
                
      Jeff Ledger (Origional PCA9555 Demo "101 COOL PROJECTS FOR THE PPDB")
      Chip Gracey (Simple Numbers - 2006)
      Jon Williams (Simple Numbers - 2006)
      Dave Fletcher (PCA9555 Driver)

    By Jorge Joaquin Pareja (http://www.WhatsAvailable.org) See website for
    contact info

    There are 2 PCA9555 I2C Bus expanders accessed in this code.  More will be
    added as the project Progresses.

    Wiring Diagram
    _____________________
    PCA9555 0 |   Propeller
    __________|__________
    SCLK      |   P0
    SDA       |   P1
    Int_PCA_0 |   P2         'Will be checked for 4x4 Keypad Press
    
    _____________________
    PCA9555 1 |   Propeller
    __________|__________
    SCLK      |   P0
    SDA       |   P1
    Int_PCA_1 |   NC         ' Never Used

    _____________________
    DS1302    |   Propeller
    __________|__________
    SCLK      |   P3
    IO        |   P4
    CE/RST    |   P5         

    _____________________
    uM-FPU V3 |   Propeller  'Not used yet , disconnected
    __________|__________
    SCLK      |   NC
    SDA       |   NC
    MCLR      |   NC

    _____________________
    Propeller |   4x4 Matrix Keypad
    __________|__________
    P8        |   4x4 Matrix Keypad Pin 1 
    P9        |   4x4 Matrix Keypad Pin 2    
    P10       |   4x4 Matrix Keypad Pin 3    
    P11       |   4x4 Matrix Keypad Pin 4    
    P12       |   4x4 Matrix Keypad Pin 5    
    P13       |   4x4 Matrix Keypad Pin 6    
    P14       |   4x4 Matrix Keypad Pin 7    
    P15       |   4x4 Matrix Keypad Pin 8 

----------------------------------
PCA9555 Device 0 ( 9 pins free )
==================================  
    _____________________
    PCA9555 0 |   17 Segment Display
    __________|__________
    IO0_0     |   R.Digit 3
    IO0_1     |   L.Digit 3
    IO0_2     |   R.Digit 2
    IO0_3     |   L.Digit 2
    IO0_4     |   R.Digit 1
    IO0_5     |   L.Digit 1
    IO0_6     |   ---------Unused
    IO0_7     |   DP  (Decimal Point)
    
   
      
----------------------------------
PCA9555 Device 1
==================================
    _____________________
    PCA9555 1 |   17 Segment Display
    __________|__________
    IO0_0     |   A1        ( In other code, A1 and A2 are reversed, not here )
    IO0_1     |   A2        ( " " )
    IO0_2     |   B 
    IO0_3     |   C 
    IO0_4     |   D1 
    IO0_5     |   D2 
    IO0_6     |   E 
    IO0_7     |   F 
    
    _____________________
    PCA9555 1 |   17 Segment Display
    __________|__________
    IO1_0     |   G1          **** G1 and G2 Differ from most wiring
    IO1_1     |   G2          **** in other OBEX code
    IO1_2     |   H
    IO1_3     |   I
    IO1_4     |   J
    IO1_5     |   K
    IO1_6     |   L
    IO1_7     |   M

----------------------------------
Propeller
==================================
    P6 to Pushbutton0   ' Connected but not used in this version

}}
CON
    _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000

    ' The following two constants are the respective PCA9555 addresses
    '
    '                       FIXED              - hardwired in all PCA9555's
    '                       ||||  A2           - hardware selectable
    '                       ||||  |A1          - hardware selectable
    '                       ||||  ||A0         - hardware selectable
    '                       ||||  |||  R/W bit *  software selectable
    '                       ||||  |||  |
    Digit                = %0100__000__0  ' LSB is r/w bit (This is PCA9555 0)
    Segment              = %0100__001__0  ' LSB is r/w bit (This is PCA9555 1)

    ' PCA9555 0's config register
    ' Digit Config
                           '
                           'Pins 8 through 1 on 4x4 matrix keypad
                           '|||| ||||  
                           '|||| |||| DP
                           '|||| |||| |unused
                           '|||| |||| ||Dgt1.L
                           '|||| |||| |||Dgt1.R
                           '|||| |||| |||| Dgt2.L
                           '|||| |||| |||| |Dgt2.R
                           '|||| |||| |||| ||Dgt3.L
                           '|||| |||| |||| |||Dgt3.R
                           '|||| |||| |||| |||| 
    DigitConfig          = %1111_0000_0100_0000 ' 0 = output, 1 = input

    ' PCA9555 1's config register
    '                     '        21   21   21
    '                     ' MLKJ IHGG FEDD CBAA
    SegmentConfig        = %0000_0000_0000_0000 ' all outputs

    ' I2C Pins on the Propeller
    SCL                  = 0
    SDA                  = 1
    KPD_INT              = 2 'Keypad interupt (from PCA9555 0) 

    ' Segment Constants 
    SegmentAllOff        = %0000_0000_0000_0000
    SegmentAllOn         = %1111_1111_1111_1111

    ' Digit Constants
    '              - High byte is 4x4 Matrix Keypad
    '              - Low byte bit 6 is for PushButton
    DigitAllOff          = %1111_0000_0100_0000
    DigitAllOn           = %1111_0000_0111_1111

    ' The I2C Expanders number of outputs
    NUM_OUTPUTS          = 16

    ' the delay, used only in PUB myWait, used for reducing code size
    DELAY                = 1000
    
    ' The number of Digits on our display
    NUM_DIGITS           = 6

    ' The respective DS1302 RTC pins on the Propeller
    RTC_Clock            = 3
    RTC_IO               = 4
    RTC_CE               = 5

    btn_ShowDate         = 6

'-------------------------
' Keypad Constants
'-------------------------

  KP_1            = %0000000000001000 ' 1
  KP_2            = %0000000000000100  ' 2
  KP_3            = %0000000000000010   ' 3
  KP_A            = %0000000000000001    ' A
  KP_4            = %0000000010000000 ' 4
  KP_5            = %0000000001000000  ' 5
  KP_6            = %0000000000100000   ' 6
  KP_B            = %0000000000010000    ' B
  KP_7            = %0000100000000000 ' 7
  KP_8            = %0000010000000000  ' 8
  KP_9            = %0000001000000000   ' 9
  KP_C            = %0000000100000000    ' C
  KP_Star         = %1000000000000000 ' *
  KP_0            = %0100000000000000  ' 0
  KP_Pound        = %0010000000000000   ' #
  KP_D            = %0001000000000000    ' D

  KP_Clock        = %1110000000000000   ' # & * (thinks 0 is also pressed, Ghosting)
    
VAR
    byte hour, minute, second, x, xx, xxx, y, yy, yyy
    byte month, day, year, dow, v, vv, vvv, w, ww, www

    WORD keypad

OBJ
   ' My math on the right is done using Parallax Propeller Tool V1.3's object info window
   ' Code size will be reduced when using PZST
   Display : "pca9555.spin"                  '                               - 101 + 2  Longs = 103
   Clock   : "DS1302.spin"                   'TODO: Start this in a new COG  - 116 + 4  Longs = 120
   SNCD    : "Simple_Numbers_CutDown.spin"   'TODO: Start this in Clock COG  - 44  + 17 Longs =  61
   KP      : "4x4 Keypad Reader"
                                            
PUB Init(startTestNow) 
    '' Setup the PCA9555's and its IO
    Display.start(SCL, SDA)

    Display.setDir(Digit, DigitConfig)
    Display.setDir(Segment, SegmentConfig)

    Display.setOut(Segment,SegmentAllOff)
    Display.setOut(Digit,DigitAllOff)

    ' Setup the ds1302 and set time
    Clock.init(RTC_Clock, RTC_IO, RTC_CE)
    Clock.config
    Clock.setDatetime( 12, 02, 11, 06, 19, 15, 00 )    ' month, day, year, day of week, hour, minute, second

    'setup the Propeller pin for reading the Interupt
    
    
    ' If this code is used as a driver, remove the following if-else lines
    if startTestNow == true
       Run(true)
    else
       Run(false)

PUB Run(runTest)
    '' Main Program Loop
    '' runTest = TRUE or FALSE.  If true, runs a display test of all segments/digits
    '' 

    'DisplayTest

    ' Main Program Loop
    repeat
      ' Get and show the time for about 12 seconds 
      repeat until keypad == KP_Clock 'ina[btn_ShowDate] == 0
        GetTime
        ShowTime
        CheckKey
      repeat until keypad == 0 'ina[btn_ShowDate] == 1
        GetDate
        ShowDate
        CheckKey


PUB CheckKey
    keypad := KP.ReadKeyPad
    
PUB GetTime 
      '' Gets the time from the ds1302
      Clock.readTime( @hour, @minute, @second )

      x := byte[(SNCD.decx(second,2))+0]
      y := byte[(SNCD.decx(second,2))+1]
      x -= "0"
      y -= "0"

      xx := byte[(SNCD.decx(minute,2))+0]
      yy := byte[(SNCD.decx(minute,2))+1]
      xx -= "0"
      yy -= "0"

      xxx := byte[(SNCD.decx(hour,2))+0]
      yyy := byte[(SNCD.decx(hour,2))+1]
      xxx -= "0"
      yyy -= "0"

PUB ShowTime      
      '' Show the time on the display left to right
      DIGITS(x,5) 'DIGIT (NUMBER,Digit)
        
      DIGITS(y,6)
      
      DIGITS(xx,3) 'DIGIT (NUMBER,Digit)
        
      DIGITS(yy,4)

      DIGITS(xxx,1) 'DIGIT (NUMBER,Digit)
        
      DIGITS(yyy,2)

PUB GetDate
      '' Get the date
      Clock.readDate( @day, @month, @year, @dow)

      v := byte[(SNCD.decx(month,2))]
      w := byte[(SNCD.decx(month,2))+1]
      v -= "0"
      w -= "0"

      vv := byte[(SNCD.decx(day,2))]
      ww := byte[(SNCD.decx(day,2))+1]
      vv -= "0"
      ww -= "0"

      vvv := byte[(SNCD.decx(year,2))]
      www := byte[(SNCD.decx(year,2))+1]
      vvv -= "0"
      www -= "0"

PUB ShowDate
      '' Show the date in MM/DD/YY format
      DIGITS(v,1) 'DIGIT (NUMBER,Digit)
        
      DIGITS(w,2)
      
      DIGITS(vv,3) 'DIGIT (NUMBER,Digit)
        
      DIGITS(ww,4)

      DIGITS(vvv,5) 'DIGIT (NUMBER,Digit)
        
      DIGITS(www,6)
      
PRI DIGITS(NUM,DIG)
    '' 
    if DIG==1
      Display.SetOut(Digit, %1111_1111_0101_1111)
    if DIG==2
      Display.SetOut(Digit, %1111_1111_0110_1111)
    if DIG==3
      Display.SetOut(Digit, %1111_1111_0111_0111)
    if DIG==4
      Display.SetOut(Digit, %1111_1111_0111_1011)
    if DIG==5
      Display.SetOut(Digit, %1111_1111_0111_1101)
    if DIG==6
      Display.SetOut(Digit, %1111_1111_0111_1110)

    if NUM==1
      Display.SetOut(Segment, ASCII_Font[49])
    if NUM==2
      Display.SetOut(Segment, ASCII_Font[50])
    if NUM==3
      Display.SetOut(Segment, ASCII_Font[51])
    if NUM==4
      Display.SetOut(Segment, ASCII_Font[52])
    if NUM==5
      Display.SetOut(Segment, ASCII_Font[53])
    if NUM==6
      Display.SetOut(Segment, ASCII_Font[54])
    if NUM==7
      Display.SetOut(Segment, ASCII_Font[55])
    if NUM==8
      Display.SetOut(Segment, ASCII_Font[56])
    if NUM==9
      Display.SetOut(Segment, ASCII_Font[57])
    if NUM==0
      Display.SetOut(Segment, ASCII_Font[48])

    DisplayOff

PUB DisplayOff
      'Display.setOut(Digit, %0000_0000_1011_1111)
      Display.setOut(Segment, SegmentAllOff)'ASCII_Font[0])

PRI myWait
    '' Used to reduce code size
    waitcnt(CNT + (CLKFREQ / DELAY))

PRI sayDone(howLong)
    '' Display Done message
    '' howLong = the amout of time to show the message
    repeat  howLong
      DisplayOff

      Display.setOut(Digit, %0000_0000_0010_1111)
      Display.setOut(Segment, ASCII_Font[68])

      DisplayOff

      Display.setOut(Digit, %0000_0000_0011_0111)
      Display.setOut(Segment, ASCII_Font[79])
      
      DisplayOff

      Display.setOut(Digit, %0000_0000_0011_1011)
      Display.setOut(Segment, ASCII_Font[78])

      DisplayOff

      Display.setOut(Digit, %0000_0000_0011_1101)
      Display.setOut(Segment, ASCII_Font[69])

    DisplayOff
      
PUB DisplayTest | i, dgts
    '' Test the PPDB's 6 digits and each segment 
    repeat dgts from 62 to 0
      Display.setOut(Digit,dgts)
      repeat i from 0 to NUM_OUTPUTS
        Display.setOut(Segment, LED_SEGMENT[i])
        myWait
      Display.setOut(Segment,SegmentAllOff)
      myWait

    sayDone(120)
      
DAT
' The following Font is different than other font maps used for the 16 Segment Display
' in other code in OBEX and the Forums.  It is setup so the index coresponds to the
' actual ASCII code weather it can be displayed on the display or not.
' lowercase might look a bit similar to Uppercase in charactors like Y and K...
'
' 
'
' Font Index 0 to 31 can be customized to your liking

                        '       21   21   21
                        'MLKJ IHGG FEDD CBAA  |  Char   |  Index/ASCII Code
                        '---------------------|---------|---------
ASCII_Font word         %0000_0000_0000_0000    '       |  0   NUL -- Null Charactor 
           word         %0000_0000_0000_0000    '       |  1   SOH -- Start of header
           word         %0000_0000_0000_0000    '       |  2   STX -- Start of text
           word         %0000_0000_0000_0000    '       |  3   ETX -- End of text
           word         %0000_0000_0000_0000    '       |  4   EOT -- End of transmission
           word         %0000_0000_0000_0000    '       |  5   ENQ -- Enquiry
           word         %0000_0000_0000_0000    '       |  6   ACK -- Acknowledgment
           word         %0000_0000_0000_0000    '       |  7   BEL -- Bell 
           word         %0000_0000_0000_0000    '       |  8   BS  -- Backspace
           word         %0000_0000_0000_0000    '       |  9   HT  -- Horizontal Tab
           word         %0000_0000_0000_0000    '       |  10  LF  -- Linefeed
           word         %0000_0000_0000_0000    '       |  11  VT  -- Vertical Tab
           word         %0000_0000_0000_0000    '       |  12  FF  -- Form Feed
           word         %0000_0000_0000_0000    '       |  13  CR  -- Carriage Return
           word         %0000_0000_0000_0000    '       |  14  SO  -- Shift Out
           word         %0000_0000_0000_0000    '       |  15  SI  -- Shift In
           word         %0000_0000_0000_0000    '       |  16  DLE -- Data link escape
           word         %0000_0000_0000_0000    '       |  17  DC1 -- Device Control 1 (XON) 
           word         %0000_0000_0000_0000    '       |  18  DC2 -- Device Control 2 
           word         %0000_0000_0000_0000    '       |  19  DC3 -- Device Control 3 (XOFF) 
           word         %0000_0000_0000_0000    '       |  20  DC4 -- Device Control 4 
           word         %0000_0000_0000_0000    '       |  21  NAK -- Negative Acknowledgment 
           word         %0000_0000_0000_0000    '       |  22  SYN -- Syncronous Idle
           word         %0000_0000_0000_0000    '       |  23  ETB -- End of transmission Block
           word         %0000_0000_0000_0000    '       |  24  CAN -- Cancel
           word         %0000_0000_0000_0000    '       |  25  EM  -- End of Medium
           word         %0000_0000_0000_0000    '       |  26  SUB -- Substitute
           word         %0000_0000_0000_0000    '       |  27  ESC -- Escape
           word         %0000_0000_0000_0000    '       |  28  FS  -- File Seperator
           word         %0000_0000_0000_0000    '       |  29  GS  -- Group Seperator
           word         %0000_0000_0000_0000    '       |  30  RS  -- Request to send
           word         %0000_0000_0000_0000    '       |  31  US  -- Unit Seperator
           '--------------------------------------------------
                        '       21   21   21
                        'MLKJ IHGG FEDD CBAA  |  Char   |  Index/ASCII Code
           word         %0000_0000_0000_0000    '       |  32  SP  -- Space  
           word         %0100_1000_0011_0000    '!      |  33  !   -- exclamation mark
           word         %0000_1000_1000_0000    '"      |  34  "   -- Double Quote
           word         %1111_1111_1111_1111    '#      |  35  #   -- Pound/number Sign
           word         %0100_1011_1011_1011    '$      |  36  $   -- Dollar Sign
           word         %0101_1011_1001_1001    '%      |  37  %   -- Percent
           word         %1101_1000_0011_1010    '&      |  38  &   -- Ampersand
           word         %0001_0000_0000_0000    '       |  39  '   -- Close Single Quote
           word         %0011_0000_0000_0000    '(      |  40  (   -- Left Parenthesis
           word         %1000_0100_0000_0000    ')      |  41  )   -- Right Parenthesis
           word         %1111_1111_0000_0000    '*      |  42  *   -- Asterisk/Star
           word         %0100_1011_0000_0000    '+      |  43  +   -- Plus
           word         %1000_0000_0000_0000    ',      |  44  ,   -- Comma
           word         %0000_0011_0000_0000    '-      |  45  -   -- Minus/Dash
           word         %0100_0001_0110_0000    '.      |  46  .   -- Period/Dot
           word         %1001_0000_0000_0000    '/      |  47  /   -- Foreward Slash
           '---------------------------------------------------
                        '       21   21   21
                        'MLKJ IHGG FEDD CBAA  |  Char   |  Index/ASCII Code
           '---------------------------------------------------
           word         %1001_0000_1111_1111    '0      |  48
           word         %0000_0000_0000_1100    '1      |  49
           word         %0000_0011_0111_0111    '2      |  50
           word         %0000_0010_0011_1111    '3      |  51
           word         %0000_0011_1000_1100    '4      |  52
           word         %0000_0011_1011_1011    '5      |  53
           word         %0000_0011_1111_1001    '6      |  54
           word         %0000_0000_0000_1111    '7      |  55
           word         %0000_0011_1111_1111    '8      |  56
           word         %0000_0011_1011_1111    '9      |  57
           '---------------------------------------------------
                        '       21   21   21
                        'MLKJ IHGG FEDD CBAA  |  Char   |  Index/ASCII Code
           '---------------------------------------------------
           word         %0100_1000_0000_0000    ':      |  58  :   -- Colon
           word         %1000_1000_0000_0000    ';      |  59  ;   -- Semicolen
           word         %0011_0000_0000_0000    '<      |  60  <   -- Less than
           word         %0000_0011_0011_0000    '=      |  61  =   -- Equal
           word         %1000_0100_0000_0000    '>      |  62  >   -- Greater than
           word         %0100_0010_0000_0110    '?      |  63  ?   -- Question mark
           word         %0000_1010_1111_0111    '@      |  64  @   -- AT symbol
           '---------------------------------------------------
                        '       21   21   21
                        'MLKJ IHGG FEDD CBAA  |  Char   |  Index/ASCII Code
           '---------------------------------------------------
           word         %0000_0011_1100_1111    'A      |  65
           word         %0100_1010_0011_1111    'B      |  66
           word         %0000_0000_1111_0011    'C      |  67
           word         %0100_1000_0011_1111    'D      |  68
           word         %0000_0011_1111_0011    'E      |  69
           word         %0000_0001_1100_0011    'F      |  70
           word         %0000_0010_1111_1011    'G      |  71
           word         %0000_0011_1100_1100    'H      |  72
           word         %0100_1000_0000_0000    'I      |  73
           word         %0000_0000_0111_1100    'J      |  74
           word         %0011_0001_0000_0000    'K      |  75
           word         %0000_0000_1111_0000    'L      |  76
           word         %0001_0100_1100_1100    'M      |  77
           word         %0010_0100_1100_1100    'N      |  78
           word         %0000_0000_1111_1111    'O      |  79
           word         %0000_0011_1100_0111    'P      |  80
           word         %0010_0000_1111_1111    'Q      |  81
           word         %0010_0011_1100_0111    'R      |  82
           word         %0010_0001_1011_0011    'S      |  83
           word         %0100_1000_0000_0011    'T      |  84
           word         %0000_0000_1111_1100    'U      |  85
           word         %1001_0000_1100_0000    'V      |  86
           word         %1010_0000_1100_1100    'W      |  87
           word         %1011_0100_0000_0000    'X      |  88
           word         %0101_0100_0000_0000    'Y      |  89
           word         %1001_0000_0011_0011    'Z      |  90
           '---------------------------------------------------
                        '       21   21   21
                        'MLKJ IHGG FEDD CBAA  |  Char   |  Index/ASCII Code
           '---------------------------------------------------
           word         %0100_1000_0010_0010    '[      |  91  [   -- Open/Left Bracket
           word         %0010_0100_0000_0000    '\      |  92  \   -- Backslash
           word         %0100_1000_0001_0001    ']      |  93  ]   -- Close/Right Bracket
           word         %1001_0000_0000_0110    '^      |  94  ^   -- Caret/circumflex
           word         %0000_0000_0011_0000    '_      |  95  _   -- Underscore
           word         %0000_0100_0000_0000    '`      |  96  `   -- Open Single Quote / Back Tick
           '---------------------------------------------------
                        '       21   21   21
                        'MLKJ IHGG FEDD CBAA  |  Char   |  Index/ASCII Code
           '---------------------------------------------------
           word         %0100_0001_0111_0000    'a      |  97
           word         %0100_0001_1110_0000    'b      |  98
           word         %0000_0001_0110_0000    'c      |  99
           word         %0100_1001_0111_0000    'd      | 100
           word         %1000_0001_0110_0000    'e      | 101
           word         %0000_0001_1100_0001    'f      | 102
           word         %0100_1000_1010_0001    'g      | 103
           word         %0100_0001_1100_0000    'h      | 104
           word         %0000_0000_0100_0001    'i      | 105
           word         %0100_1000_0110_0000    'j      | 106
           word         %0011_0001_1100_0000    'k      | 107
           word         %0100_1000_0000_0000    'l      | 108
           word         %0100_0011_0100_1000    'm      | 109
           word         %0100_0001_0100_0000    'n      | 110
           word         %0100_0001_0110_0000    'o      | 111
           word         %0000_1001_1100_0001    'p      | 112
           word         %0100_1001_1001_1001    'q      | 113
           word         %0101_0000_0000_0000    'r      | 114
           word         %0100_0010_0010_0000    's      | 115
           word         %0100_0011_0000_0000    't      | 116
           word         %0100_0000_0110_0000    'u      | 117
           word         %0000_0000_0110_0000    'v      | 118
           word         %0100_0000_0111_1000    'w      | 119
           word         %1010_0011_0000_0000    'x      | 120
           word         %1000_1001_0000_0000    'y      | 121
           word         %1000_0001_0010_0000    'z      | 122
           '---------------------------------------------------
                        '       21   21   21
                        'MLKJ IHGG FEDD CBAA  |  Char   |  Index/ASCII Code
           '---------------------------------------------------
           word         %0100_1001_0001_0010    '{      | 123 {    -- Open/Left Brace
           word         %0100_1000_0000_0000    '|      | 124 |    -- Verticle Bar / Pipe
           word         %0100_1010_0010_0001    '}      | 125 }    -- Close/Right Brace
           word         %0000_0011_0100_0100    '~      | 126 ~    -- Tilde
           word         %0000_1110_0000_0111    '       | 127 DEL  -- Delete

DAT
' Each Segment of the LED Display in order from A1 to M
           '---------------------------------------------------
                        '       21   21   21
                        'MLKJ IHGG FEDD CBAA  | Segment | Dec  | Hex
           '---------------------------------------------------
LED_SEGMENT word        %0000_0000_0000_0000    ' None  |     0 |    0
            word        %0000_0000_0000_0001    ' A1    |     1 |    1
            word        %0000_0000_0000_0010    ' A2    |     2 |    2
            word        %0000_0000_0000_0100    ' B     |     4 |    4
            word        %0000_0000_0000_1000    ' C     |     8 |    8
            word        %0000_0000_0001_0000    ' D1    |    16 |   10
            word        %0000_0000_0010_0000    ' D2    |    32 |   20
            word        %0000_0000_0100_0000    ' E     |    64 |   40
            word        %0000_0000_1000_0000    ' F     |   128 |   80
            word        %0000_0001_0000_0000    ' G1    |   256 |  100
            word        %0000_0010_0000_0000    ' G2    |   512 |  200
            word        %0000_0100_0000_0000    ' H     |  1024 |  400
            word        %0000_1000_0000_0000    ' I     |  2048 |  800
            word        %0001_0000_0000_0000    ' J     |  4096 | 1000
            word        %0010_0000_0000_0000    ' K     |  8192 | 2000
            word        %0100_0000_0000_0000    ' L     | 16384 | 4000
            word        %1000_0000_0000_0000    ' M     | 32768 | 8000
DAT
' Each digit from Left to Right (6 total) default, 0, is all inputs

LED_DIGIT   word        %1111_1111_1111_1111    ' None  | 65535 | FFFF   
            word        %1111_1111_1101_1111    ' D1.L  | 65503 | FFDF
            word        %1111_1111_1110_1111    ' D1.R  | 65519 | FFEF
            word        %1111_1111_1111_0111    ' D2.L  | 65527 | FFF7
            word        %1111_1111_1111_1011    ' D2.R  | 65531 | FFFB
            word        %1111_1111_1111_1101    ' D3.L  | 65533 | FFFD
            word        %1111_1111_1111_1110    ' D3.R  | 65534 | FFFE
                                                        