{{

  PCA9555 Driver By Dave Fletcher
  A .600 mil DIP-24 version of this chip currently costs $1.66 US from DigiKey.

  Based on Minimal I2C driver 2008-01 by deSilva
  http://forums.parallax.com/forums/default.aspx?f=25&m=244743            

}}
CON
  INPORT0  = 0 ' Input port 0
  INPORT1  = 1 ' Input port 1
  OUTPORT0 = 2 ' Output port 0
  OUTPORT1 = 3 ' Output port 1
  POLINV0  = 4 ' Polarity Inversion port 0
  POLINV1  = 5 ' Polarity Inversion port 1
  CONF0    = 6 ' Configuration port 0
  CONF1    = 7 ' Configuration port 1

VAR
  LONG scl
  LONG sda

PUB start(_scl, _sda)
  scl := _scl
  sda := _sda

PUB setDir(addr, value)
  return writecmd(addr, CONF0, CONF1, value)

PUB getDir(addr)
  return readcmd(addr, CONF0, CONF1)

PUB setOut(addr, value)   
  return writecmd(addr, OUTPORT0, OUTPORT1, value)

PUB getOut(addr)
  return readcmd(addr, OUTPORT0, OUTPORT1) 
  
PUB setPolInv(addr, value) : recvd
  return writecmd(addr, POLINV0, POLINV1, value)

PUB getPolInv(addr)
  return readcmd(addr, POLINV0, POLINV1) 
  
PUB getIn(addr) : recvd
  return readcmd(addr, INPORT0, INPORT1) 

PRI writecmd(addr, cmdlow, cmdhigh, value) : status | high, low
  status := 0
  high := (value & %1111_1111_0000_0000) >> 8 
  low  := value & %0000_0000_1111_1111
  status := _writecmd(addr, cmdlow, low)
  status |= _writecmd(addr, cmdhigh, high) 
    
PRI _writecmd(addr, command, value) : status
  engage 
  status := snd(addr)
  status |= snd(command)
  status |= snd(value)
  release

PRI readcmd(addr, cmdlow, cmdhigh) : retval
  retval := _readcmd(addr, cmdhigh)
  retval <<= 8
  retval |= _readcmd(addr, cmdlow)

PRI _readcmd(addr, command) : retval | status, high, low
  retval := 0
  engage 
  status := snd(addr)
  status |= snd(command)
  engage 
  status |= snd(addr+1)
  retval := rcv
  release
    
PRI engage
  ' Pulls sda to LOW whilst CLK is HIGH
   dira[scl]~~                     
   outa[sda]~  
  ' The former two instructions need be done only once, if all is o.k.
   outa[scl]~~   
   dira[sda]~    ' Pulse sda; HIGH by pullup (default)
   dira[sda]~~   ' Enable = LOW
   outa[scl]~    ' Keep scl LOW
  
PRI snd(by) : status 
  status := 0
  ' Send a byte (and get status: 0: ACK  1: NAK)
  by := (!by) >< 8             ' Reverse bits and invert pattern
  repeat 8       
    dira[sda] := by
    outa[scl]~~                ' Transfer on rising edge
    by >>= 1
    outa[scl]~ 
  ' Returns "acknowledge bit": 0 = ACK, 1 = NAK.
  dira[sda]~            ' release sda
  outa[scl]~~           ' puls-in ACK
  status := ina[sda]
  outa[scl]~            ' keep CLK low
  
PRI release
  ' Assert stop condition.
  ' sda transits from LOW to HIGH whilst scl is HIGH
  dira[sda]~~           ' LOW
  outa[scl]~~
  dira[sda]~            ' release sda (HIGH)
   
PRI rcv : val
  val := 0
  ' Get a byte
  repeat 8
    outa[scl]~~        ' Request bit
    val <<= 1
    val |= ina[sda]
    outa[scl]~         ' Keep CLK low

PRI nak
  ' Send ackbit
  dira[sda]~~           ' LOW
  outa[scl]~~           ' Pulse scl
  outa[scl]~            ' keep CLK low
  dira[sda]~            ' release sda
  
PRI ack
  ' Send ackbit
  dira[sda]~          ' HIGH
  outa[scl]~~         ' Pulse scl
  outa[scl]~          ' keep CLK low 
  dira[sda]~          ' release sda